@@ -1,6 +1,6 @@
 package HTML::Element;
 
-# $Id: Element.pm,v 1.32 1996/05/26 10:29:48 aas Exp $
+# $Id: Element.pm,v 1.37 1997/12/02 12:48:09 aas Exp $
 
 =head1 NAME
 
@@ -24,13 +24,15 @@ HTML::Element - Class for objects that represent HTML elements
 =head1 DESCRIPTION
 
 Objects of the HTML::Element class can be used to represent elements
-of HTML.  These objects have attributes and content.  The content is a
-sequence of text segments and other HTML::Element objects.  Thus a
+of HTML.  These objects have attributes and content.  The content is an
+array of text segments and other HTML::Element objects.  Thus a
 tree of HTML::Element objects as nodes can represent the syntax tree
 for a HTML document.
 
 The following methods are available:
 
+=over 4
+
 =cut
 
 
@@ -42,7 +44,7 @@ use vars qw($VERSION
 	    %emptyElement %optionalEndTag %linkElements %boolean_attr
            );
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.32 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
 sub Version { $VERSION; }
 
 # Elements that does not have corresponding end tags (i.e. are empty)
@@ -50,7 +52,7 @@ sub Version { $VERSION; }
 			             img br hr wbr
 			             input area param
 			            );
-%optionalEndTag = map { $_ => 1 } qw(p li dt dd option th tr td);
+%optionalEndTag = map { $_ => 1 } qw(p li dt dd option); # th tr td);
 
 # Elements that might contain links and the name of the link attribute
 %linkElements =
@@ -86,22 +88,22 @@ sub Version { $VERSION; }
  ul     => 'compact',
 );
 
-=head2 $h = HTML::Element->new('tag', 'attrname' => 'value',...)
+=item $h = HTML::Element->new('tag', 'attrname' => 'value',...)
 
-The object constructor.  Takes an tag name as argument. Optionally
+The object constructor.  Takes a tag name as argument. Optionally,
 allows you to specify initial attributes at object creation time.
 
 =cut
 
 #
 # An HTML::Element is represented by blessed hash reference.  Key-names
-# not starting with '_' is reserved for the SGML attributes of the element.
+# not starting with '_' are reserved for the SGML attributes of the element.
 # The following special keys are used:
 #
 #    '_tag':    The tag name
 #    '_parent': A reference to the HTML::Element above (when forming a tree)
-#    '_pos':    The current position (a reference to a HTML::Element), is
-#               where inserts will placed (look at the insert_element method)
+#    '_pos':    The current position (a reference to a HTML::Element) is
+#               where inserts will be placed (look at the insert_element method)
 #
 # Example: <img src="gisle.jpg" alt="Gisle's photo"> is represented like this:
 #
@@ -131,7 +133,7 @@ sub new
 
 
 
-=head2 $h->tag()
+=item $h->tag()
 
 Returns (optionally sets) the tag name for the element.  The tag is
 always converted to lower case.
@@ -150,7 +152,7 @@ sub tag
 
 
 
-=head2 $h->starttag()
+=item $h->starttag()
 
 Returns the complete start tag for the element.  Including leading
 "<", trailing ">" and attributes.
@@ -169,8 +171,17 @@ sub starttag
 	    exists($boolean_attr{$name}) && $boolean_attr{$name} eq $_) {
 	    $tag .= " \U$_";
 	} else {
-	    HTML::Entities::encode_entities($val, '&">');
-	    $val = qq{"$val"} unless $val =~ /^\d+$/;
+	    if ($val !~ /^\d+$/) {
+		# count number of " compared to number of '
+		if (($val =~ tr/\"/\"/) > ($val =~ tr/\'/\'/)) {
+		    # use single quotes around the attribute value
+		    HTML::Entities::encode_entities($val, "&'>");
+		    $val = qq('$val');
+		} else {
+		    HTML::Entities::encode_entities($val, '&">');
+		    $val = qq{"$val"};
+		}
+	    }
 	    $tag .= qq{ \U$_\E=$val};
 	}
     }
@@ -179,9 +190,9 @@ sub starttag
 
 
 
-=head2 $h->endtag()
+=item $h->endtag()
 
-Returns the complete end tag.  Including leading "</" and the trailing
+Returns the complete end tag.  Includes leading "</" and the trailing
 ">".
 
 =cut
@@ -193,7 +204,7 @@ sub endtag
 
 
 
-=head2 $h->parent([$newparent])
+=item $h->parent([$newparent])
 
 Returns (optionally sets) the parent for this element.
 
@@ -211,7 +222,7 @@ sub parent
 
 
 
-=head2 $h->implicit([$bool])
+=item $h->implicit([$bool])
 
 Returns (optionally sets) the implicit attribute.  This attribute is
 used to indicate that the element was not originally present in the
@@ -226,7 +237,7 @@ sub implicit
 
 
 
-=head2 $h->is_inside('tag',...)
+=item $h->is_inside('tag',...)
 
 Returns true if this tag is contained inside one of the specified tags.
 
@@ -248,7 +259,7 @@ sub is_inside
 
 
 
-=head2 $h->pos()
+=item $h->pos()
 
 Returns (and optionally sets) the current position.  The position is a
 reference to a HTML::Element object that is part of the tree that has
@@ -272,7 +283,7 @@ sub pos
 
 
 
-=head2 $h->attr('attr', [$value])
+=item $h->attr('attr', [$value])
 
 Returns (and optionally sets) the value of some attribute.
 
@@ -291,10 +302,11 @@ sub attr
 
 
 
-=head2 $h->content()
+=item $h->content()
 
 Returns the content of this element.  The content is represented as a
-array of text segments and references to other HTML::Element objects.
+reference to an array of text segments and references to other
+HTML::Element objects.
 
 =cut
 
@@ -305,7 +317,7 @@ sub content
 
 
 
-=head2 $h->is_empty()
+=item $h->is_empty()
 
 Returns true if there is no content.
 
@@ -319,7 +331,7 @@ sub is_empty
 
 
 
-=head2 $h->insert_element($element, $implicit)
+=item $h->insert_element($element, $implicit)
 
 Inserts a new element at current position and updates pos() to point
 to the inserted element.  Returns $element.
@@ -348,7 +360,7 @@ sub insert_element
 }
 
 
-=head2 $h->push_content($element_or_text,...)
+=item $h->push_content($element_or_text,...)
 
 Adds to the content of the element.  The content should be a text
 segment (scalar) or a reference to a HTML::Element object.
@@ -379,7 +391,7 @@ sub push_content
 
 
 
-=head2 $h->delete_content()
+=item $h->delete_content()
 
 Clears the content.
 
@@ -397,9 +409,9 @@ sub delete_content
 
 
 
-=head2 $h->delete()
+=item $h->delete()
 
-Frees memory assosiated with the element an all children.  This is
+Frees memory associated with the element and all children.  This is
 needed because perl's reference counting does not work since we use
 circular references.
 
@@ -416,15 +428,15 @@ sub delete
 
 
 
-=head2 $h->traverse(\&callback, [$ignoretext])
+=item $h->traverse(\&callback, [$ignoretext])
 
-Traverse the element and all its children.  For each node visited, the
+Traverse the element and all of its children.  For each node visited, the
 callback routine is called with the node, a startflag and the depth as
 arguments.  If the $ignoretext parameter is true, then the callback
 will not be called for text content.  The flag is 1 when we enter a
 node and 0 when we leave the node.
 
-If the return value from the callback is false then we will not
+If the returned value from the callback is false then we will not
 traverse the children.
 
 =cut
@@ -449,9 +461,9 @@ sub traverse
 
 
 
-=head2 $h->extract_links([@wantedTypes])
+=item $h->extract_links([@wantedTypes])
 
-Returns links found by traversing the element and all its children.
+Returns links found by traversing the element and all of its children.
 The return value is a reference to an array.  Each element of the
 array is an array with 2 values; the link value and a reference to the
 corresponding element.
@@ -493,7 +505,7 @@ sub extract_links
 
 
 
-=head2 $h->dump()
+=item $h->dump()
 
 Prints the element and all its children to STDOUT.  Mainly useful for
 debugging.  The structure of the document is shown by indentation (no
@@ -519,7 +531,7 @@ sub dump
 
 
 
-=head2 $h->as_HTML()
+=item $h->as_HTML()
 
 Returns a string (the HTML document) that represents the element and
 its children.
@@ -565,6 +577,7 @@ sub format
 
 __END__
 
+=back
 
 =head1 BUGS
 
@@ -582,13 +595,9 @@ L<HTML::AsSubs>
 
 =head1 COPYRIGHT
 
-Copyright 1995,1996 Gisle Aas.  All rights reserved.
+Copyright 1995-1997 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
-=head1 AUTHOR
-
-Gisle Aas <aas@sn.no>
-
 =cut
