@@ -1,11 +1,17 @@
 package HTML::Formatter;
 
-# $Id: Formatter.pm,v 1.12 1996/04/09 15:44:13 aas Exp $
+# $Id: Formatter.pm,v 1.19 1998/01/06 09:51:02 aas Exp $
 
 =head1 NAME
 
 HTML::Formatter - Base class for HTML formatters
 
+=head1 SYNOPSIS
+
+ package HTML::FormatXX;
+ require HTML::Formatter;
+ @ISA=qw(HTML::Formatter);
+
 =head1 DESCRIPTION
 
 HTML formatters are able to format a HTML syntax tree into various
@@ -20,7 +26,7 @@ L<HTML::FormatText>, L<HTML::FormatPS>, L<HTML::Element>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1995 Gisle Aas. All rights reserved.
+Copyright (c) 1995-1997 Gisle Aas. All rights reserved.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -36,11 +42,31 @@ require HTML::Element;
 
 use strict;
 use Carp;
+use UNIVERSAL qw(can);
+
+use vars qw($VERSION);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.19 $ =~ /(\d+)\.(\d+)/);
 
 sub new
 {
-    my $class = shift;
-    bless { }, $class;
+    my($class,%arg) = @_;
+    my $self = bless { $class->default_values }, $class;
+    $self->configure(\%arg) if scalar(%arg);
+    $self;
+}
+
+sub default_values
+{
+    ();
+}
+
+sub configure
+{
+    my($self, $arg) = @_;
+    for (keys %$arg) {
+	warn "Unknown configure argument '$_'" if $^W;
+    }
+    $self;
 }
 
 sub format
@@ -53,7 +79,13 @@ sub format
 	    if (ref $node) {
 		my $tag = $node->tag;
 		my $func = $tag . '_' . ($start ? "start" : "end");
-		return $self->$func($node);
+		# Use UNIVERSAL::can so that we can recover if
+		# a handler is not defined for the tag.
+		if (can($self, $func)) {
+		    return $self->$func($node);
+		} else {
+		    return 1;
+		}
 	    } else {
 		$self->textflow($node);
 	    }
@@ -79,9 +111,8 @@ sub begin
     $self->{font_size}     = [3];   # last element is current size
     $self->{basefont_size} = [3];
 
-    $self->{makers} = [];           # last element is current marker
-    $self->{vspace} = undef;        # vertical space
-    $self->{eat_leading_space} = 0;
+    $self->{markers} = [];          # last element is current marker
+    $self->{vspace} = undef;        # vertical space (dimension)
 
     $self->{output} = [];
 }
@@ -130,16 +161,13 @@ sub h6_end   { shift->header_end(6, @_) }
 sub br_start
 {
     my $self = shift;
-    $self->vspace(0);
-    $self->eat_leading_space;
-
+    $self->vspace(0, 1);
 }
 
 sub hr_start
 {
     my $self = shift;
     $self->vspace(1);
-    $self->eat_leading_space;
 }
 
 sub img_start
@@ -233,7 +261,7 @@ sub font_start
 {
     my($self, $elem) = @_;
     my $size = $elem->attr('size');
-    return unless defined $size;
+    return 1 unless defined $size;
     if ($size =~ /^\s*[+\-]/) {
 	my $base = $self->{basefont_size}[-1];
 	$size = $base + $size;
@@ -289,7 +317,6 @@ sub p_start
 {
     my $self = shift;
     $self->vspace(1);
-    $self->eat_leading_space;
     1;
 }
 
@@ -324,7 +351,6 @@ sub blockquote_start
 {
     my $self = shift;
     $self->vspace(1);
-    $self->eat_leading_space;
     $self->adjust_lm( +2 );
     $self->adjust_rm( -2 );
     1;
@@ -342,7 +368,6 @@ sub address_start
 {
     my $self = shift;
     $self->vspace(1);
-    $self->eat_leading_space;
     $self->i_start(@_);
     1;
 }
@@ -378,7 +403,6 @@ sub li_start
     my $self = shift;
     $self->bullet($self->{markers}[-1]);
     $self->adjust_lm(+2);
-    $self->eat_leading_space;
     1;
 }
 
@@ -444,7 +468,6 @@ sub dt_start
 {
     my $self = shift;
     $self->vspace(1);
-    $self->eat_leading_space;
     1;
 }
 
@@ -457,7 +480,6 @@ sub dd_start
     my $self = shift;
     $self->adjust_lm(+6);
     $self->vspace(0);
-    $self->eat_leading_space;
     1;
 }
 
@@ -477,14 +499,15 @@ sub textflow
 {
     my $self = shift;
     if ($self->{pre}) {
+	# strip leading and trailing newlines so that the <pre> tags 
+	# may be placed on lines of their own without causing extra
+	# vertical space as part of the preformatted text
+	$_[0] =~ s/\n$//;
+	$_[0] =~ s/^\n//;
 	$self->pre_out($_[0]);
     } else {
 	for (split(/(\s+)/, $_[0])) {
 	    next unless length $_;
-	    if ($self->{eat_leading_space}) {
-		$self->{eat_leading_space} = 0;
-		next if /^\s/;
-	    }
 	    $self->out($_);
 	}
     }
@@ -492,17 +515,19 @@ sub textflow
 
 
 
-sub eat_leading_space
-{
-    shift->{eat_leading_space} = 1;
-}
-
-
 sub vspace
 {
-    my($self, $new) = @_;
-    return if defined $self->{vspace} and $self->{vspace} > $new;
-    $self->{vspace} = $new;
+    my($self, $min, $add) = @_;
+    my $old = $self->{vspace};
+    if (defined $old) {
+	my $new = $old;
+	$new += $add || 0;
+	$new = $min if $new < $min;
+	$self->{vspace} = $new;
+    } else {
+	$self->{vspace} = $min;
+    }
+    $old;
 }
 
 sub collect
