@@ -1,9 +1,9 @@
 #
-# $Id: LWP.pm,v 1.62 1997/12/16 19:19:20 aas Exp $
+# $Id: LWP.pm,v 1.66 1998/01/20 15:11:55 aas Exp $
 
 package LWP;
 
-$VERSION = "5.18_04";
+$VERSION = "5.18_05";
 sub Version { $VERSION; }
 
 require 5.004;
@@ -32,7 +32,7 @@ allow you to write WWW clients, thus libwww-perl said to be a WWW
 client library. The library also contain modules that are of more
 general use.
 
-The main architecture of the library is object oriented.  The user
+Most modules in this library are object oriented.  The user
 agent, requests sent and responses received from the WWW server are
 all represented by objects.  This makes a simple and powerful
 interface to these services.  The interface should be easy to extend
@@ -50,7 +50,7 @@ used separately or together.
 =item *
 
 Provides an object oriented model of HTTP-style communication.  Within
-this framework we currently support access to http, gopher, ftp, news,
+this framework we currently support access to http, https, gopher, ftp, news,
 file, and mailto resources.
 
 =item *
@@ -96,6 +96,10 @@ scripts).
 
 =item *
 
+It can deal with HTTP cookies.
+
+=item *
+
 A simple command line client application called C<lwp-request>.
 
 =back
@@ -278,16 +282,6 @@ address will be sent to the servers with every request.
 
 =item *
 
-The B<use_alarm> specify if it is OK for the user agent to use the
-alarm(2) system to implement timeouts.
-
-=item *
-
-The B<use_eval> specify if the agent should raise an
-exception (C<die> in perl) if an error condition occur.
-
-=item *
-
 The B<parse_head> specify whether we should initialize response
 headers from the E<lt>head> section of HTML documents.
 
@@ -352,7 +346,9 @@ $ua->from value.
 For all responses, the library will add a header called "Client-Date".
 This header will encode the time when the response was received by
 your application.  This format and semantics of the header is just
-like the server created "Date" header.
+like the server created "Date" header.  You can also encounter other
+"Client-XXX" headers.  They are all generated by the library
+internally and not something really passed on from the servers.
 
 =head2 HTTP Requests
 
@@ -384,6 +380,21 @@ through the HTTP protocol module.  See L<LWP::UserAgent> for
 documentation of this.
 
 
+=head2 HTTPS Requests
+
+HTTPS requests are HTTP requests over an encrypted network connection
+using the SSL protocol developed by Netscape.  Everything about HTTP
+requests above also hold for HTTPS requests.  In addition the library
+will add the headers "Client-SSL-Cipher", "Client-SSL-Cert-Subject" and
+"Client-SSL-Cert-Issuer" to the response.  These headers denote the
+encryption method used and the name of the server owner.
+
+The request can contain the header "If-SSL-Cert-Subject" in order to
+make the request conditional on the content of the server certificate.
+If the certificate subject does not match, no request is sent to the
+server and an internally generated error response is returned.  The
+value of the "If-SSL-Cert-Subject" header is interpreted as a Perl
+regular expression.
 
 
 =head2 FTP Requests
@@ -463,7 +474,7 @@ encoded (as the first letter) in the request URL path itself.
 
 Example:
 
-  $req = HTTP::Request->new('GET', 'gopher://gopher.sn.no/');
+  $req = HTTP::Request->new(GET => 'gopher://gopher.sn.no/');
 
 
 
@@ -492,7 +503,7 @@ The "To" header is initialized from the mail address in the URL.
 Example:
 
   $req = HTTP::Request->new(POST => 'mailto:libwww-perl-request@ics.uci.edu');
-  $req->header("Subject", "subscribe");
+  $req->header(Subject => "subscribe");
   $req->content("Please subscribe me to the libwww-perl mailing list!\n");
 
 
@@ -510,7 +521,6 @@ library. Indentation shows class inheritance.
      LWP::Protocol::ftp   -- ftp:// access
      ...
 
- LWP::Socket        -- Socket creation and IO
  LWP::Authen::Basic -- Handle 401 and 407 responses
  LWP::Authen::Digest
 
@@ -546,12 +556,6 @@ The following modules provide various functions and definitions.
  HTML::Entities     -- Expand or unexpand entities in HTML text
  File::Listing      -- Parse directory listings
 
-HTTP use the Base64 encoding at some places.  The QuotedPrint module
-is just included to make the MIME:: collection more complete.
-
- MIME::Base64       -- Base64 encoding/decoding routines
- MIME::QuotedPrint  -- Quoted Printable encoding/decoding routines
-
 The following modules does not have much to do with the World Wide
 Web, but are included just because I am lazy and did not bother to
 make separate distributions for them.  Regard them as bonus, provided
@@ -564,7 +568,7 @@ free for your pleasure.
 =head1 MORE DOCUMENTATION
 
 All modules contain detailed information on the interfaces they
-provide.  The L<lwpcook> is the libwww-perl cookbook that contain
+provide.  The I<lwpcook> manpage is the libwww-perl cookbook that contain
 examples of typical usage of the library.  You might want to take a
 look at how the scripts C<lwp-request>, C<lwp-rget> and C<lwp-mirror>
 are implemented.
@@ -594,7 +598,7 @@ Koenig, Jared Rhine, and Jack Shirazi.
 
 =head1 COPYRIGHT
 
-  Copyright 1995-1997, Gisle Aas
+  Copyright 1995-1998, Gisle Aas
   Copyright 1995, Martijn Koster
 
 This library is free software; you can redistribute it and/or
